==> vertsms_0.4.tar.gz: vertsms-0.4/src/opt/vertsms/sms.py <==
#!/usr/bin/env python2.5

import os, sqlite3, time
import dbus
import ctypes, uuid, glib, gobject



# Information taken from:
# http://docs.python.org/library/uuid.html#uuid.uuid4
# http://docs.python.org/library/ctypes.html
# http://www.dalkescientific.com/writings/NBN/ctypes.html
# Class constructed from: http://maemo.gitorious.org/maemo-rtcom/rtcom-eventlogger/blobs/master/rtcom-eventlogger/event.h
# credits to gri for doing the hard work with this


class RTComElEvent(ctypes.Structure):
    _fields_ = [("_mask", ctypes.c_int),
                ("fld_id", ctypes.c_int),
                ("fld_service_id", ctypes.c_int),
                ("fld_event_type_id", ctypes.c_int),
                ("fld_storage_time", ctypes.c_long), # time_t
                ("fld_start_time", ctypes.c_long), # time_t
                ("fld_end_time", ctypes.c_long), # time_t
                ("fld_is_read", ctypes.c_int), # Boolean is int
                ("fld_flags", ctypes.c_int),
                ("fld_bytes_sent", ctypes.c_int),
                ("fld_bytes_received", ctypes.c_int),
                ("fld_local_uid", ctypes.c_char_p),
                ("fld_local_name", ctypes.c_char_p),
                ("fld_remote_uid", ctypes.c_char_p),
                ("fld_remote_name", ctypes.c_char_p),
                ("fld_remote_ebook_uid", ctypes.c_char_p),
                ("fld_channel", ctypes.c_char_p),
                ("fld_free_text", ctypes.c_char_p),
                ("fld_group_uid", ctypes.c_char_p),
                ("fld_service", ctypes.c_char_p),
                ("fld_event_type", ctypes.c_char_p),
                ("fld_additional_text", ctypes.c_char_p),
                ("fld_icon_name", ctypes.c_char_p),
                ("fld_pango_markup", ctypes.c_char_p)]




def put_to_sent_sms(name, number, uniqueid, groupid, message):
#    rtcom = ctypes.CDLL('librtcom-eventlogger.so.1') # PR 1.2
    rtcom = ctypes.CDLL('librtcom-eventlogger.so.0') # PR 1.1

    el = ctypes.cast(rtcom.rtcom_el_new(), ctypes.c_void_p)
    event = ctypes.cast(rtcom.rtcom_el_event_new(), ctypes.POINTER(RTComElEvent))[0]

    event._mask       |= 1 << 18
    event.fld_service  = 'RTCOM_EL_SERVICE_SMS'

    event._mask       |= 1 << 19
    event.fld_event_type = 'RTCOM_EL_EVENTTYPE_SMS_OUTBOUND'

    event._mask       |= 1 << 4
    event.fld_start_time = int(time.time())

    event._mask       |= 1 << 5
    event.fld_end_time = int(time.time())

    event._mask       |= 1 << 6
    event.fld_is_read = 1 # TRUE

    event._mask       |= 1 << 10
    event.fld_remote_ebook_uid = uniqueid

    event._mask       |= 1 << 11
    event.fld_local_uid = 'ring/tel/ring'

    event._mask       |= 1 << 12
    event.fld_local_name = '<SelfHandle>'

    event._mask       |= 1 << 13
    event.fld_remote_uid = number

    event._mask       |= 1 << 14
    event.fld_remote_name = name

    event._mask       |= 1 << 17
    event.fld_group_uid = groupid

    event._mask       |= 1 << 16
    event.fld_free_text = message

    print "adding event..."
    rtcom.rtcom_el_add_event.argtypes = [el, ctypes.c_void_p, ctypes.c_void_p]
    eventId = rtcom.rtcom_el_add_event(el, ctypes.byref(event), None)
    print "added event: %s" % eventId


    # the follwing block doesnt seem to be needed for SMS
    # Add the message that it shows up in conversations
    rtcom.rtcom_el_add_header.argtypes = [el, ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_void_p]
    rtcom.rtcom_el_add_header(el, eventId, 'message-token', str(uuid.uuid4()), None)

    rtcom.rtcom_el_event_free(ctypes.byref(event))
    #glib.g_object_unref(el)
    



class SMS(object):
    '''
    Sends sms messages
    '''

    def __init__(self, msg, number):
        self.pdustring = self.createPDUstring(number.replace('+', '00'), msg)
        self.write_to_rtcomm(msg, number)

    def send(self):
        self.__dbus_send(self.pdustring)

    def __dbus_send(self, pdu_string):
        bus = dbus.SystemBus()
        smsobject = bus.get_object('com.nokia.phone.SMS', '/com/nokia/phone/SMS/ba212ae1')
        smsiface = dbus.Interface(smsobject, 'com.nokia.csd.SMS.Outgoing')
        arr = dbus.Array(pdu_string)
        msgdeliver = dbus.Array([arr])
        smsiface.Send(msgdeliver,'')


    def octify(self, str):
        bytes = map(ord, str)
        bitsconsumed = 0
        referencebit = 7
        octets = []
        bitstocopy = 0

        while len(bytes):
            byte = bytes.pop(0)
            byte = byte >> bitsconsumed

            try:
                nextbyte = bytes[0]

                bitstocopy = (nextbyte & (0xff >> referencebit)) << referencebit
                octet = (byte | bitstocopy)

            except:
                octet = (byte | 0x00)

            if bitsconsumed != 7:
                octets.append(byte | bitstocopy)
                bitsconsumed += 1
                referencebit -= 1
            else:
                bitsconsumed = 0
                referencebit = 7

        return octets

    def semi_octify(self, str):
        '''
        Expects a string containing two digits.
        Returns an octet -
        first nibble in the octect is the first
        digit and the second nibble represents
        the second digit.
        '''

        try:
            digit_1 = int(str[0])
            digit_2 = int(str[1])
            octet = (digit_2 << 4) | digit_1
        except:
            octet = (1 << 4) | digit_1

        return octet

    def resetnumber(self, number):
        '''
        Adds trailing F to number if length is
        odd.
        '''
        length = len(number)
        if (length % 2) != 0:
            number = number + 'F'

        return number

    def createPDUstring(self, number, msg):
        '''
        Returns a list of bytes to represent a valid PDU message
        '''

        octifiedmsg = self.octify(msg)
        number = self.resetnumber(number)
        octifiednumber = [ self.semi_octify(number[i:i+2]) for i in range(0, len(number), 2) ]

        HEADER = 1
        FIRSTOCTETOFSMSDELIVERMSG = 10
        ADDR_TYPE = 129 # unknown format
        number_length = len(number)
        msg_length = len(msg)
        pdu_message = [HEADER, FIRSTOCTETOFSMSDELIVERMSG, number_length, ADDR_TYPE]
        pdu_message.extend(octifiednumber)
        pdu_message.append(0)
        pdu_message.append(0)
        pdu_message.append(msg_length)
        pdu_message.extend(octifiedmsg)

        return pdu_message

    def write_to_rtcomm(self, message, number):
        connection = sqlite3.connect(os.path.join(os.environ['HOME'],
                                     '.rtcom-eventlogger', 'el.db'))
        cursor = connection.cursor()        
        cursor.execute('SELECT id FROM Events')
        id_array = cursor.fetchall()  
        seq_id = max(id_array)[0] + 1
        
        strippednumber = number[-7:]

        own_number = ""
        
	put_to_sent_sms(own_number, number, 0, strippednumber, message)

==> vertsms_0.5.tar.gz: vertsms-0.5/src/opt/vertsms/sms.py <==
#!/usr/bin/env python2.5

import os, sqlite3, time
import dbus
import ctypes, uuid, glib, gobject



# Information taken from:
# http://docs.python.org/library/uuid.html#uuid.uuid4
# http://docs.python.org/library/ctypes.html
# http://www.dalkescientific.com/writings/NBN/ctypes.html
# Class constructed from: http://maemo.gitorious.org/maemo-rtcom/rtcom-eventlogger/blobs/master/rtcom-eventlogger/event.h
# credits to gri for doing the hard work with this


class RTComElEvent(ctypes.Structure):
    _fields_ = [("_mask", ctypes.c_int),
                ("fld_id", ctypes.c_int),
                ("fld_service_id", ctypes.c_int),
                ("fld_event_type_id", ctypes.c_int),
                ("fld_storage_time", ctypes.c_long), # time_t
                ("fld_start_time", ctypes.c_long), # time_t
                ("fld_end_time", ctypes.c_long), # time_t
                ("fld_is_read", ctypes.c_int), # Boolean is int
                ("fld_flags", ctypes.c_int),
                ("fld_bytes_sent", ctypes.c_int),
                ("fld_bytes_received", ctypes.c_int),
                ("fld_local_uid", ctypes.c_char_p),
                ("fld_local_name", ctypes.c_char_p),
                ("fld_remote_uid", ctypes.c_char_p),
                ("fld_remote_name", ctypes.c_char_p),
                ("fld_remote_ebook_uid", ctypes.c_char_p),
                ("fld_channel", ctypes.c_char_p),
                ("fld_free_text", ctypes.c_char_p),
                ("fld_group_uid", ctypes.c_char_p),
                ("fld_service", ctypes.c_char_p),
                ("fld_event_type", ctypes.c_char_p),
                ("fld_additional_text", ctypes.c_char_p),
                ("fld_icon_name", ctypes.c_char_p),
                ("fld_pango_markup", ctypes.c_char_p)]




def put_to_sent_sms(name, number, uniqueid, groupid, message):
    rtcom = ctypes.CDLL('librtcom-eventlogger.so.1') # PR 1.2
#    rtcom = ctypes.CDLL('librtcom-eventlogger.so.0') # PR 1.1

    el = ctypes.cast(rtcom.rtcom_el_new(), ctypes.c_void_p)
    event = ctypes.cast(rtcom.rtcom_el_event_new(), ctypes.POINTER(RTComElEvent))[0]

    event._mask       |= 1 << 18
    event.fld_service  = 'RTCOM_EL_SERVICE_SMS'

    event._mask       |= 1 << 19
    event.fld_event_type = 'RTCOM_EL_EVENTTYPE_SMS_OUTBOUND'

    event._mask       |= 1 << 4
    event.fld_start_time = int(time.time())

    event._mask       |= 1 << 5
    event.fld_end_time = int(time.time())

    event._mask       |= 1 << 6
    event.fld_is_read = 1 # TRUE

    event._mask       |= 1 << 10
    event.fld_remote_ebook_uid = uniqueid

    event._mask       |= 1 << 11
    event.fld_local_uid = 'ring/tel/ring'

    event._mask       |= 1 << 12
    event.fld_local_name = '<SelfHandle>'

    event._mask       |= 1 << 13
    event.fld_remote_uid = number

    event._mask       |= 1 << 14
    event.fld_remote_name = name

    event._mask       |= 1 << 17
    event.fld_group_uid = groupid

    event._mask       |= 1 << 16
    event.fld_free_text = message

    print "adding event..."
    rtcom.rtcom_el_add_event.argtypes = [el, ctypes.c_void_p, ctypes.c_void_p]
    eventId = rtcom.rtcom_el_add_event(el, ctypes.byref(event), None)
    print "added event: %s" % eventId


    # the follwing block doesnt seem to be needed for SMS
    # Add the message that it shows up in conversations
    rtcom.rtcom_el_add_header.argtypes = [el, ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_void_p]
    rtcom.rtcom_el_add_header(el, eventId, 'message-token', str(uuid.uuid4()), None)

    rtcom.rtcom_el_event_free(ctypes.byref(event))
    #glib.g_object_unref(el)
    



class SMS(object):
    '''
    Sends sms messages
    '''

    def __init__(self, msg, number):
#        print number.find("+")
        if number.find("+") > -1:
          conv_number = number.replace('+', '')
          print "international number"
          self.ADDR_TYPE = 129
#        change ADDR_TYPE ?!? 129 = international, 4 = local
#        print self.pdustring
        else:
          conv_number = number
          self.ADDR_TYPE = 132
        self.pdustring = self.createPDUstring(conv_number, msg)
        self.write_to_rtcomm(msg, number)

    def send(self):
        self.__dbus_send(self.pdustring)

    def __dbus_send(self, pdu_string):
        bus = dbus.SystemBus()
        smsobject = bus.get_object('com.nokia.phone.SMS', '/com/nokia/phone/SMS/ba212ae1')
        smsiface = dbus.Interface(smsobject, 'com.nokia.csd.SMS.Outgoing')
        arr = dbus.Array(pdu_string)
        msgdeliver = dbus.Array([arr])
        smsiface.Send(msgdeliver,'')


    def octify(self, str):
        bytes = map(ord, str)
        bitsconsumed = 0
        referencebit = 7
        octets = []
        bitstocopy = 0

        while len(bytes):
            byte = bytes.pop(0)
            byte = byte >> bitsconsumed

            try:
                nextbyte = bytes[0]

                bitstocopy = (nextbyte & (0xff >> referencebit)) << referencebit
                octet = (byte | bitstocopy)

            except:
                octet = (byte | 0x00)

            if bitsconsumed != 7:
                octets.append(byte | bitstocopy)
                bitsconsumed += 1
                referencebit -= 1
            else:
                bitsconsumed = 0
                referencebit = 7
#        print "octet: " + str(octets)
        return octets

    def semi_octify(self, str):
        '''
        Expects a string containing two digits.
        Returns an octet -
        first nibble in the octect is the first
        digit and the second nibble represents
        the second digit.
        '''

        try:
            digit_1 = int(str[0])
            digit_2 = int(str[1])
            octet = (digit_2 << 4) | digit_1
        except:
            octet = (1 << 4) | digit_1
#        print "semioctet: " + str(octet)
        return octet

    def resetnumber(self, number):
        '''
        Adds trailing F to number if length is
        odd.
        '''
        length = len(number)
        if (length % 2) != 0:
            number = number + 'F'

        return number

    def createPDUstring(self, number, msg):
        '''
        Returns a list of bytes to represent a valid PDU message
        '''

        octifiedmsg = self.octify(msg)
        number = self.resetnumber(number)
        octifiednumber = [ self.semi_octify(number[i:i+2]) for i in range(0, len(number), 2) ]

        HEADER = 5 #was 1
#        HEADER = 1
        FIRSTOCTETOFSMSDELIVERMSG = 10
#        ADDR_TYPE = 129 # unknown format
        number_length = len(number)
        msg_length = len(msg)
        pdu_message = [HEADER, FIRSTOCTETOFSMSDELIVERMSG, number_length, self.ADDR_TYPE]
        pdu_message.extend(octifiednumber)
        pdu_message.append(0)
        pdu_message.append(0)
        pdu_message.append(msg_length)
        pdu_message.extend(octifiedmsg)

        return pdu_message

    def write_to_rtcomm(self, message, number):
        connection = sqlite3.connect(os.path.join(os.environ['HOME'],
                                     '.rtcom-eventlogger', 'el.db'))
        cursor = connection.cursor()        
        cursor.execute('SELECT id FROM Events')
        id_array = cursor.fetchall()  
        seq_id = max(id_array)[0] + 1
        
        strippednumber = number[-7:]

        own_number = ""
        
	put_to_sent_sms(own_number, number, 0, strippednumber, message)
